//+------------------------------------------------------------------+
//|                                               RAVI FX Fisher.mq4 |
//|                         Copyright  2005, Luis Guilherme Damiani |
//|                                      http://www.damianifx.com.br |
//+------------------------------------------------------------------+
#property copyright "Copyright  2005, Luis Guilherme Damiani"
#property link      "http://www.damianifx.com.br"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 PaleVioletRed
#property indicator_color2 LimeGreen
#property indicator_color3 Silver
#property indicator_color4 Silver
//---- input parameters

extern string TimeFrame        = "Current time frame";
extern int    MAfast           = 4;
extern int    MAslow           = 49;
extern double trigger          = 0.07;
extern bool   alertsOn         = False;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;
extern bool   Show3StateMode   = true;

//---- buffers
double RAVIfxFishBuffer[];
double LoTrigBuff[];
double HiTrigBuff[];
double LoNeutBuff[];
double HiNeutBuff[];
double trend[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,LoTrigBuff); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,HiTrigBuff); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,LoNeutBuff); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,HiNeutBuff); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,RAVIfxFishBuffer);
   SetIndexBuffer(5,trend);
   
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue";   if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";       if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //

   return(0);
}
int deinit()
{
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
  {
   int counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { LoTrigBuff[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      for (int shift = limit; shift>=0; shift--)
      {
         double maf = iMA(NULL,0,MAfast,0,MODE_LWMA,PRICE_TYPICAL,shift);
         double mas = iMA(NULL,0,MAslow,0,MODE_LWMA,PRICE_TYPICAL,shift);
         double atf = iATR(NULL,0,MAfast,shift);
         double ats = iATR(NULL,0,MAslow,shift);
         double ma  = 0;
            if (mas!=0 && ats!=0)
	          ma = 100 * ((maf-mas)*atf)/mas/ats;
	           double IFish=(MathExp(2*ma)-1)/(MathExp(2*ma)+1);
	    
         //
         //
         //
         //
         //
         
         RAVIfxFishBuffer[shift] = IFish;
         LoTrigBuff[shift]       = EMPTY_VALUE;
         HiTrigBuff[shift]       = EMPTY_VALUE;
         LoNeutBuff[shift]       = EMPTY_VALUE;
         HiNeutBuff[shift]       = EMPTY_VALUE;
         trend[shift]            = trend[shift+1];
            if (RAVIfxFishBuffer[shift]> trigger)                                        trend[shift] =  1;
            if (RAVIfxFishBuffer[shift]<-trigger)                                        trend[shift] = -1;
            if (RAVIfxFishBuffer[shift]< trigger && RAVIfxFishBuffer[shift]> (-trigger)) trend[shift] =  0;
            if (Show3StateMode)
            {
               if  (trend[shift]== 1) { LoTrigBuff[shift] = Low[shift]; HiTrigBuff[shift] = High[shift]; }
               if  (trend[shift]==-1) { HiTrigBuff[shift] = Low[shift]; LoTrigBuff[shift] = High[shift]; }
               if  (trend[shift]== 0) { HiNeutBuff[shift] = Low[shift]; LoNeutBuff[shift] = High[shift]; }
            }
            else
            {
               if  (RAVIfxFishBuffer[shift]>0) { LoTrigBuff[shift] = Low[shift]; HiTrigBuff[shift] = High[shift]; }
               if  (RAVIfxFishBuffer[shift]<0) { HiTrigBuff[shift] = Low[shift]; LoTrigBuff[shift] = High[shift]; }
            }
       }
       manageAlerts();    
       return(0);   
   }
   
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (shift=limit; shift>=0; shift--)
   {
      int y = iBarShift(NULL,timeFrame,Time[shift]);
      trend[shift]      = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",MAfast,MAslow,trigger,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,5,y);
      LoTrigBuff[shift] = EMPTY_VALUE;
      HiTrigBuff[shift] = EMPTY_VALUE;
      LoNeutBuff[shift] = EMPTY_VALUE;
      HiNeutBuff[shift] = EMPTY_VALUE;
            if (Show3StateMode)
            {
               if  (trend[shift]== 1) { LoTrigBuff[shift] = Low[shift]; HiTrigBuff[shift] = High[shift]; }
               if  (trend[shift]==-1) { HiTrigBuff[shift] = Low[shift]; LoTrigBuff[shift] = High[shift]; }
               if  (trend[shift]== 0) { HiNeutBuff[shift] = Low[shift]; LoNeutBuff[shift] = High[shift]; }
            }
            else
            {
               if  (RAVIfxFishBuffer[shift]>0) { LoTrigBuff[shift] = Low[shift]; HiTrigBuff[shift] = High[shift]; }
               if  (RAVIfxFishBuffer[shift]<0) { HiTrigBuff[shift] = Low[shift]; LoTrigBuff[shift] = High[shift]; }
            }
   }
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1)                           doAlert(whichBar,"crossed trigger level up");
         if (trend[whichBar] == -1)                           doAlert(whichBar,"crossed trigger level down");
         if (trend[whichBar] == 0 && trend[whichBar+1] ==  1) doAlert(whichBar,"reverted from upper zone down");
         if (trend[whichBar] == 0 && trend[whichBar+1] == -1) doAlert(whichBar,"reverted from lower zone up");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  timeFrameToString(timeFrame)+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" RAVI fx fisher "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," RAVI fx fisher "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}